@@ -23,13 +23,15 @@ class ResourceLoader(object):
             cls._instance._trie_tree = None
             cls._instance._crf_seg_model = None
             cls._instance._crf_pos_model = None
+            cls._instance._idf_table=None
             cls._instance._break_table = None
             cls._instance._break_regex_method = None
             cls._instance._combine_regex_method = None
         return cls._instance
 
-    def load_crf_seg_model(self, path=None, **options):
-        if not self._crf_seg_model:
+    def load_crf_seg_model(self, path=None, force=False):
+        if not self._crf_seg_model or force:
+            options = {}
             if path:
                 options['model'] = path
             else:
@@ -46,8 +48,9 @@ class ResourceLoader(object):
             self._crf_seg_model = _crf_seg_model
         return self._crf_seg_model
 
-    def load_crf_pos_model(self, path=None, **options):
-        if not self._crf_pos_model:
+    def load_crf_pos_model(self, path=None, force=False):
+        if not self._crf_pos_model or force:
+            options = {}
             if path:
                 options['model'] = path
             else:
@@ -64,8 +67,8 @@ class ResourceLoader(object):
             self._crf_pos_model = _crf_pos_model
         return self._crf_pos_model
 
-    def load_trie_tree(self, path=None):
-        if not self._trie_tree:
+    def load_trie_tree(self, path=None, force=False):
+        if not self._trie_tree or force:
             trie_tree = TrieTree()
             if not path:
                 path = library_path
@@ -86,8 +89,24 @@ class ResourceLoader(object):
             self._trie_tree = trie_tree
         return self._trie_tree
 
-    def load_break_table(self, path=None):
-        if not self._break_table:
+    def load_idf_table(self, path=None, force=False):
+        if not self._idf_table or force:
+            if not path:
+                idf_path = os.path.join(library_path, "idf.txt")
+            else:
+                idf_path = path 
+            tree = {}
+            if not os.path.exists(idf_path):
+                return
+            with open(idf_path, 'rb') as idf_file:
+                for line in idf_file:
+                    label = line.decode("utf8").strip().split('\t')
+                    tree[label[0]] = float(label[1])
+            self._idf_table = tree
+        return self._idf_table
+
+    def load_break_table(self, path=None, force=False):
+        if not self._break_table or force:
             if not path:
                 break_idx = os.path.join(library_path, "break.txt")
             else:
@@ -102,8 +121,8 @@ class ResourceLoader(object):
             self._break_table = tree
         return self._break_table
 
-    def load_break_regex_method(self, path=None):
-        if not self._break_regex_method:
+    def load_break_regex_method(self, path=None, force=False):
+        if not self._break_regex_method or force:
             _break_regex_list = []
             if not path:
                 break_regex_path = os.path.join(library_path, "break.regex")
@@ -118,14 +137,12 @@ class ResourceLoader(object):
             pattern = u'|'.join(
                 [u'[%s]+[*?]*' % regex for regex in _break_regex_list])
             pattern += u'|[^%s]+[*?]*' % u''.join(_break_regex_list)
-            self._break_regex_method = re.compile(
-                pattern,
-                re.UNICODE,
-            ).findall
+            regex_compile = re.compile(pattern, re.UNICODE)
+            self._break_regex_method = regex_compile.findall
         return self._break_regex_method
 
-    def load_combine_regex_method(self, path=None):
-        if not self._combine_regex_method:
+    def load_combine_regex_method(self, path=None, force=False):
+        if not self._combine_regex_method or force:
             _combine_regex_list = []
             if not path:
                 combine_regex_path = os.path.join(
@@ -138,8 +155,15 @@ class ResourceLoader(object):
                     if not regex or regex.startswith('#'):
                         continue
                     _combine_regex_list.append(regex)
-            self._combine_regex_method = re.compile(
-                '|'.join(_combine_regex_list),
-                re.UNICODE,
-            ).match
+            regex_compile = re.compile('|'.join(_combine_regex_list), re.UNICODE)
+            """
+            fix '^a$' regex can match 'a\n'
+            """
+            def _combine_regex_method(text):
+                m = regex_compile.match(text)
+                if m and m.group() == text:
+                    return True
+                else:
+                    return False
+            self._combine_regex_method = _combine_regex_method
         return self._combine_regex_method
